<?php
namespace Service;

use Util\Utilities;
use Zeedhi\Framework\Session\Session;
use \Service\Structure;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Manager\Mongo\Manager;
use \Service\Indicator;
use \Service\Widget;
use \Util\Environment;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class View {

    protected $dataSourceName = 'view';
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $organizationManager;
     /** @var \Zeedhi\Framework\Session\Session $sessionManager  */
    protected $sessionManager;
    /** @var \Service\Indicator $indicatorService */
    protected $indicatorService;
    /** @var \Service\Widget $widgetService */
    protected $widgetService;
    /** @var \Util\Environment $environment */
    protected $environment;


    const VIEW_TYPE_DASHBOARD = 'D';
    const VIEW_TYPE_MENU = 'M';
    const VIEW_TYPE_GENERATED_MENU = 'G';

    public function __construct(Manager $organizationManager, Session $sessionManager, Indicator $indicatorService,
                                Widget $widgetService, Environment $environment) {
        $this->organizationManager = $organizationManager;
        $this->sessionManager = $sessionManager;
        $this->indicatorService = $indicatorService;
        $this->widgetService = $widgetService;
        $this->environment = $environment;
        $this->organizationManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findBy(FilterCriteria $filterCriteria) {        
        $filterCriteria->setPageSize(99999);
        $filterCriteria->setPage(1);
        return $this->organizationManager->findByPagination($filterCriteria, $filterCriteria->getOrderBy())->getRows();
    }

    public function findNext ($params) {
        $response = $this->organizationManager->findNext($this->dataSourceName, $params);
        return $response;
    }

    public function findById($dataSource) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(array(
            "columnName"=>"_id",
            "value" => $dataSource,
            "operator" => "="
        )));

        return $this->organizationManager->findBy($filterCriteria)->getRows();
    }

    public function count($filter) {
	    $filterCriteria = new ZmartFilterCriteria($filter->getDataSourceName(), $filter->getConditions());
	    $filterCriteria->setPage(1);
	    $filterCriteria->setPageSize(9999);
	    return $this->organizationManager->count($filterCriteria);
	}

    public function setLog($dataBefore, $id)
    {
        $dataAfter = $this->findById($id);
        Utilities::setLog($dataBefore, $dataAfter, 'dashboard', $id);
    }

    public function findDashboardViewsByMenu($menuId, $menuType) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);

        $accessProfiles = $this->sessionManager->get('accessProfiles');

        if ($menuType === self::VIEW_TYPE_MENU){
            $filterCriteria->addCondition("parent", $menuId);
        } else if ($menuType === self::VIEW_TYPE_GENERATED_MENU) {
            $filterCriteria->addCondition("_id", FilterCriteria::EQ, $menuId);
        }

        $filterCriteria->addCondition("type", self::VIEW_TYPE_DASHBOARD);
        $dashboard = $this->findBy($filterCriteria);
        $dashboardsWithAccess = array();
        if(!empty($dashboard) && !empty($accessProfiles)){
            foreach($dashboard as $dash){if(!is_array($dash['accessProfiles'])){$dash['accessProfiles']=array($dash['accessProfiles']);}
                if(array_intersect($dash['accessProfiles'],$accessProfiles) || \Util\Utilities::isTeknisa()){
                    array_push($dashboardsWithAccess,$dash);
                } else {
                    array_push($dashboardsWithAccess,array("error" => true, "name" => $dash['name']));
                }
            }
        }
        return $dashboardsWithAccess;
    }

    public function getMainDashboardViews($name = null, $id = null) {
        return $this->getDashboardViews($name, true, $id);
    }

    public function getDashboardViews($name = null, $isMain = false, $id = null) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $accessProfiles = $this->sessionManager->get('accessProfiles');
        if(empty($accessProfiles)){
            $accessProfiles = $this->sessionManager->get('ACCESS_PROFILE');
        }
        $dbName = $this->organizationManager->getDbName();
        if($accessProfiles && $dbName != Utilities::getImportDefaultDatabase()) {
            if(!$this->environment->getNrOrgClient()){
                $filterCriteria->addCondition("accessProfiles", FilterCriteria::IN, $accessProfiles);
            }else if($this->environment->getNrOrgClient() == $this->environment->getOrganizationId()){
                $filterCriteria->addCondition("accessProfiles", FilterCriteria::IN, $accessProfiles);
            }

        }
        if($id){
            $filterCriteria->addCondition("_id", $id);
        }else if($name){
            $filterCriteria->addCondition("name", $name);
        }
        if($isMain) {
            $filterCriteria->addCondition("isMainDashboard", true);
        }
        
        $filterCriteria->addCondition("type", self::VIEW_TYPE_DASHBOARD);
        $filterCriteria->addOrderBy('order', FilterCriteria::ORDER_ASC);
        $mainDashboardViews = $this->findBy($filterCriteria);
        return $mainDashboardViews;
    }

    public function getGenericDashboardViews($name = null, $idDashboard = null, $showHiddenViews = false, $isMobile = false) {
        $userId = InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
        $userViews = $this->getUserViews($userId,$idDashboard,$isMobile);
        if ($showHiddenViews) {
            return [$this->getDashboardViews($name, false, null), $userViews];
        }
        return [$this->filterHiddenViews($this->getDashboardViews($name, false, null), $userViews, true), $userViews];
    }

    protected function filterHiddenViews($dashboardViews, $userViews, $notHidden) {
        $hiddenViews = array_values(array_filter(\Util\Utilities::arrayKeysRecursive(array_column($userViews, "hidden")), function($value) {
            return strlen($value) > 1;
        }));
        if (!empty($hiddenViews)){
            $filteredIndicators = array_values(array_filter($dashboardViews[0]["indicators"], function($view) use ($hiddenViews, $notHidden) {
                if ($notHidden) {
                    return !in_array($view["widget"], $hiddenViews);
                } else {
                    return in_array($view["widget"], $hiddenViews);
                }
            }));
            $dashboardViews[0]["indicators"] = $filteredIndicators;
        }
        return $dashboardViews;
    }

    public function getHiddenViews(){
        $userId = InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
        $userViews = $this->getUserViews($userId);
        return $this->filterHiddenViews($this->getDashboardViews($name, false, null), $userViews);
    }

    public function loadMenuHierarchy() {
        $views = $this->getViewsByLoggedUser();
        $indicators = $this->getIndicatorsByLoggedUser();
        $menuHierarchyItems = $this->buildMenuHierarchyItems($views, $indicators);
        $this->buildDashboardHierarchyItems($views, $menuHierarchyItems);

        return array_values($menuHierarchyItems);
    }

    protected function buildMenuHierarchyItems(array $views, array $indicators) {
        $menuItems = array(); 
        $this->buildMenuHierarchyItemsFromViews($views, $menuItems);
        $this->buildMenuHierarchyItemsFromIndicators($indicators, $menuItems);
        return $menuItems;
    }

    protected function buildMenuHierarchyItemsFromViews($views, &$menuItems) {
        foreach ($views as $view) {
            if ($view["type"] === self::VIEW_TYPE_MENU) {
                $menuItem = $this->factoryHierarchyItem($view["_id"], $view["name"], isset($view['order']) ? $view['order'] : null);
                $menuItem["parent"] = empty($view["parent"]) ? null : $view["parent"];
                $menuItem["indicators"] = array();
                $menuItem["hasDashboards"] = false;

                $indicators = $this->indicatorService->findByMenu($view["_id"]);
                foreach ($indicators as $indicator) {
                    $menuItem["indicators"][] = $this->buildIndicatorItem($indicator, $menuItem);
                }
                $menuItems[$menuItem['_id']] = $menuItem;
            }
        }
    }

    protected function buildMenuHierarchyItemsFromIndicators($indicators, &$menuItems) {
        foreach ($indicators as $indicator) {
            $menuItem = $this->factoryHierarchyItem($indicator["_id"], $indicator["name"], isset($indicator['order']) ? $indicator['order'] : null);
            $menuItem["hasDashboards"] = false;

            $indicatorItem = $this->buildIndicatorItem($indicator, $menuItem);
            $indicatorItem['ignoreFirstMenuLevel'] = true;
            $menuItem["indicators"] = isset($indicatorItem) ? array($indicatorItem) : array();
            $menuItems[$menuItem['_id']] = $menuItem;
        }
    }

    protected function buildIndicatorItem($indicator) {
        $indicatorItem = $this->factoryHierarchyItem($indicator["_id"], $indicator["name"], isset($indicator['order']) ? $indicator['order'] : null);
        $indicatorItem["views"] = array();

        $widgets = $this->widgetService->findWidgetsByIndicator($indicator['_id']);
        foreach ($widgets as $widget) {
            $indicatorItem["views"][] = $this->factoryHierarchyItem($widget["_id"], $widget["label"], isset($widget['order']) ? $widget['order'] : null);
        }

        return $indicatorItem;
    }

    protected function buildDashboardHierarchyItems(array $views, &$menuHierarchy) {
        foreach ($views as $view) {
            $type = isset($view["type"]) ? $view["type"] : null;
            $parent = isset($view["parent"]) ? $view["parent"] : null;
            if ($type === self::VIEW_TYPE_DASHBOARD && $parent && isset($menuHierarchy[$parent])) {
                $menuHierarchy[$parent]["hasDashboards"] = true;
            }
        }
    }

    protected function factoryHierarchyItem($id, $name, $order = null) {
        return array(
            "_id" => $id,
            "name" => $name,
            "order" => $order
        );
    }

    public function resetUserViews($viewId, $userId, $isMobile = false) {
        $filters = [
            array("columnName"=>"userId", "value" => $userId, "operator" => FilterCriteria::EQ),
            array("columnName"=>"viewId", "value" => $viewId, "operator" => FilterCriteria::EQ)
        ];
        $userView = $this->findBy(new FilterCriteria("userView", $filters));
        if(!empty($userView) && !empty($userView[0]) && !empty($userView[0]["_id"])){
            $dataSet = array(
                "userId" => $userId,
                "viewId" => $viewId,
                "data" => [],
                "isMobile" => $isMobile,
                "_id" => $userView[0]["_id"]
            );
            $dataSet = new DataSet('userView', [$dataSet]);
            $this->organizationManager->persist($dataSet);
        }
    }

    public function saveUserView($viewId, $userId, $data, $isMobile = false) {
        $dataSet = array(
            "userId" => $userId,
            "viewId" => $viewId,
            "data" => $data,
            "isMobile" => $isMobile,
        );
        $filters = [
            array("columnName"=>"userId", "value" => $userId, "operator" => FilterCriteria::EQ),
            array("columnName"=>"viewId", "value" => $viewId, "operator" => FilterCriteria::EQ)
        ];
        $userView = $this->findBy(new FilterCriteria("userView", $filters));
        if(!empty($userView) && !empty($userView[0]) && !empty($userView[0]["_id"])){
            $dataSet["_id"] = $userView[0]["_id"];
        }
        $dataSet = new DataSet('userView', [$dataSet]);
        $this->organizationManager->persist($dataSet);
    }

    public function saveViewIndicatorPositions($viewId, $data) {
        $filters = [
            array("columnName"=>"_id", "value" => $viewId, "operator" => FilterCriteria::EQ),
        ];
        $view = $this->findBy(new FilterCriteria("view", $filters));
        if(!empty($view) && !empty($view[0]) && !empty($view[0]["indicators"])){
            foreach($view[0]["indicators"] as &$dbIndicator) {
                foreach($data as $newIndicator) {
                    if($newIndicator["i"] == $dbIndicator["widget"]) {
                        $dbIndicator["x"] = $newIndicator["x"];
                        $dbIndicator["y"] = $newIndicator["y"];
                        $dbIndicator["width"] = $newIndicator["w"];
                        $dbIndicator["height"] = $newIndicator["h"];
                    }
                }
            }
            $dataSet = new DataSet('view', $view);
            $this->organizationManager->persist($dataSet);
        }
    }

    public function getUserViews($userId,$idDashboard = null, $isMobile = false) {
        $filters = [
            array("columnName"=>"userId", "value" => $userId, "operator" => FilterCriteria::EQ),
        ];
        if ($isMobile) {
            $filters[] = array("columnName"=>"isMobile", "value" => true, "operator" => FilterCriteria::EQ);
        } else {
            $filters[] = array("columnName"=>"isMobile", "value" => true, "operator" => FilterCriteria::NEQ);
        }
        if ($idDashboard){
            $filters[] = array("columnName"=>"viewId", "value" => $idDashboard, "operator" => FilterCriteria::EQ);
        }
        $userViewDb = $this->findBy(new FilterCriteria("userView", $filters));
        $userViews = [];
        foreach ($userViewDb as $userView) {
            $dataIndexed = [
                "hidden" => []
            ];
            foreach ($userView["data"] as $data){
                if (!empty($data)){
                    if(empty($data["i"])){
                        continue;
                    }
                    $parameters = [
                        "w" => !empty($data["w"]) ? round($data["w"]) : 0,
                        "h" => !empty($data["h"]) ? round($data["h"]) : 0,
                        "x" => !empty($data["x"]) ? round($data["x"]) : 0,
                        "y" => !empty($data["y"]) ? round($data["y"]) : 0
                    ];
                    if (empty($data["hidden"])){
                        $dataIndexed[$data["i"]] = $parameters;
                    } else {
                        $dataIndexed["hidden"][$data["i"]] = $parameters;
                    }
                }
            }
            $userViews[$userView["viewId"]] = $dataIndexed;
        }
        return $userViews;
    }

    public function getViewsByLoggedUser() {
        $views = array();
        $accessProfiles = $this->sessionManager->get('accessProfiles');
        if(\Util\Utilities::isTeknisa()){
            $views = $this->findBy(new FilterCriteria($this->dataSourceName));
            $this->removeAuditing($views);
        }
        else if (!empty($accessProfiles)) {
            $filters = array(
                array("columnName"=>"accessProfiles", "value" => $accessProfiles, "operator" => FilterCriteria::IN)
            );
            $views = $this->findBy(new FilterCriteria($this->dataSourceName, $filters));
            $this->removeAuditing($views);
        }
        return $views;
    }

    public function getIndicatorsByLoggedUser() {
        $nullValues = array(null, "");
        $filters = array(
            array("columnName"=>"menu", "value" => $nullValues, "operator" => FilterCriteria::IN),
            array("columnName"=>"showInMenu", "value" => true, "operator" => "=")
        );
        $indicators = $this->findBy(new FilterCriteria('indicator', $filters));
        return $indicators;
    }

    protected function removeAuditing(&$views) {
        foreach ($views as $i => $view) {
            unset($views[$i]['accessProfiles']);
            unset($views[$i]['created_by']);
            unset($views[$i]['created_by_name']);
        }
    }
}
